@@ -1,14 +1,17 @@
 package Toadfarm;
 use Mojo::Base 'Mojolicious';
-use Mojo::Util qw( class_to_path monkey_patch );
+
 use Cwd 'abs_path';
 use Data::Dumper ();
-use File::Basename qw( basename dirname );
+use File::Basename qw(basename dirname);
 use File::Spec;
 use File::Which;
+use Mojo::File;
+use Mojo::Util qw(class_to_path monkey_patch);
+
 use constant DEBUG => $ENV{TOADFARM_DEBUG} ? 1 : 0;
 
-our $VERSION = '0.75';
+our $VERSION = '0.76';
 
 BEGIN {
   $ENV{TOADFARM_ACTION} //= (@ARGV and $ARGV[0] =~ /^(reload|start|stop)$/) ? $1 : 'load';
@@ -28,13 +31,14 @@ sub import {
   unshift @{$app->commands->namespaces}, 'Toadfarm::Command';
 
   monkey_patch $caller, (
-    app     => sub {$app},
+    app         => sub {$app},
+    change_root => \&_change_root,
     logging => sub { $tf->{logging}++; $app->_setup_log(@_) },
-    mount   => sub { push @{$app->config->{apps}}, @_ == 2 ? @_ : ($_[0], {}); $app },
-    plugin  => sub { push @{$app->config->{plugins}}, @_ == 2 ? @_ : ($_[0], {}); $app },
-    run_as  => \&_run_as,
+    mount  => sub { push @{$app->config->{apps}},    @_ == 2 ? @_ : ($_[0], {}); $app },
+    plugin => sub { push @{$app->config->{plugins}}, @_ == 2 ? @_ : ($_[0], {}); $app },
+    run_as => \&_run_as,
     secrets => sub { $tf->{secrets}++; $app->secrets([@_]) },
-    start   => sub {
+    start => sub {
       if (@_) {
         my $listen = ref $_[0] eq 'ARRAY' ? shift : undef;
         $app->config->{hypnotoad} = @_ > 1 ? {@_} : {%{$_[0]}} if @_;
@@ -69,6 +73,23 @@ sub startup {
   $self->_mount_root_app(delete $self->{root_app}) if $self->{root_app};
 }
 
+sub _change_root {
+  my @cmd  = @_;
+  my $exit = -2;
+
+  return 1 if $<;    # not root
+
+  unshift @cmd, $ENV{TOADFARM_CHROOT_BIN} || 'chroot';
+  push @cmd, $^X;
+  push @cmd, -I => $INC[0] if $ENV{TOADFARM_ACTION} eq 'test';
+  push @cmd, File::Spec->rel2abs($0), @ARGV;
+
+  warn "[Toadfarm] system @cmd\n" if DEBUG;
+  system @cmd;
+  die "Could not run '@cmd' exit=$exit\n" if $exit = $? >> 8;
+  exit $?;
+}
+
 sub _die_on_insecure {
   my ($class, $app) = @_;
   my $config = $app->config;
@@ -118,13 +139,12 @@ sub _mount_apps {
       require File::Temp;
       my %config = (%{$self->config}, %{$rules->{config}});
       $tmp = File::Temp->new;
-      Mojo::Util::spurt(
+      Mojo::File->new($tmp->filename)->spurt(
         do {
           local $Data::Dumper::Terse    = 1;
           local $Data::Dumper::Deepcopy = 1;
           Data::Dumper::Dumper(\%config);
-        },
-        $tmp->filename
+          }
       );
       $ENV{MOJO_CONFIG} = $tmp->filename;
     }
@@ -147,7 +167,7 @@ sub _mount_apps {
 
     $app->config->{$_} ||= $config->{$_} for keys %$config;
 
-    for my $k (qw( local_port remote_address remote_port )) {
+    for my $k (qw(local_port remote_address remote_port)) {
       push @over, $self->_skip_if(tx => $k, delete $rules->{$k});
     }
 
@@ -186,7 +206,7 @@ sub _mount_root_app {
 sub _paths {
   my ($self, $config) = @_;
 
-  for my $type (qw( renderer static )) {
+  for my $type (qw(renderer static)) {
     my $paths = $config->{$type} or next;
     $self->$type->paths($paths);
   }
@@ -212,7 +232,7 @@ sub _run_as {
   for my $p (File::Spec->path) {
     $sudo[0] = File::Spec->catfile($p, 'sudo');
     next unless -x $sudo[0];
-    push @sudo, qw( -i -n -u ), "#$uid";
+    push @sudo, qw(-i -n -u), "#$uid";
     last;
   }
 
@@ -288,7 +308,7 @@ Toadfarm - One Mojolicious app to rule them all
 
 =head1 VERSION
 
-0.75
+0.76
 
 =head1 DESCRIPTION
 
